'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = transformPostCSS;

var _path = require('path');

var _child_process = require('child_process');

// note: socket path is important to keep short as it will be truncated if it
// exceeds certain platform limits. for this reason, we're writing to /tmp
// instead of using os.tmpdir (which can, on platforms like darwin, be quite
// long & per-process).
const projectId = process.cwd().toLowerCase().replace(/[^a-z]/g, '');
const socketName = `bptp-${ projectId }.sock`;
const socketPath = (0, _path.join)('/tmp', socketName);

const usingBabel = process.argv.indexOf('babel-register') !== -1;
const babel = (0, _path.join)(__dirname, '..', 'node_modules', '.bin', 'babel-node');
const nodeExecutable = !usingBabel ? process.argv[0] : babel;
const clientExcutable = (0, _path.join)(__dirname, 'postcss-client.js');
const serverExcutable = (0, _path.join)(__dirname, 'postcss-server.js');

const startServer = (() => {
  let server;

  return () => {
    if (server) {
      return;
    }

    server = (0, _child_process.spawn)(nodeExecutable, [serverExcutable, socketPath], {
      env: process.env, // eslint-disable-line no-process-env
      stdio: 'inherit'
    });

    server.unref();
    process.on('exit', () => server.kill());
  };
})();

function transformPostCSS({ types: t }) {
  const extensions = ['.css'];

  return {
    visitor: {
      CallExpression(path, { file, opts }) {
        const { callee: { name: calleeName }, arguments: args } = path.node;

        if (calleeName !== 'require' || !args.length || !t.isStringLiteral(args[0])) {
          return;
        }

        const [{ value: stylesheetPath }] = args;
        const stylesheetExtension = (0, _path.extname)(stylesheetPath);

        if (extensions.indexOf(stylesheetExtension) !== -1) {
          startServer();

          const requiringFile = file.opts.filename;
          const cssFile = (0, _path.resolve)((0, _path.dirname)(requiringFile), stylesheetPath);
          const configFile = (0, _path.resolve)(opts.config);
          const data = JSON.stringify({ cssFile, configFile });
          const execArgs = [clientExcutable, socketPath, data];
          const result = (0, _child_process.execFileSync)(nodeExecutable, execArgs, {
            env: process.env }).toString('utf8');
          const tokens = JSON.parse(result || '{}');

          path.replaceWith(new t.ObjectExpression(Object.keys(tokens).map(token => new t.ObjectProperty(new t.StringLiteral(token), new t.StringLiteral(tokens[token])))));
        }
      }
    }
  };
}